package aexp.oneway;

import android.app.Activity;
import android.content.Intent;
import android.content.ComponentName;
import android.content.ServiceConnection;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

public class OneWayActivity extends Activity
{
    private static final String LOG_TAG = "ONEWAYACTIVITY";

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        initService();
        Button getCounterButton = (Button)findViewById( R.id.getcounter );
        getCounterButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
                if( counterService != null ) {
                    try {
                        int counter = counterService.getCounter();
                        TextView counterTextView = (TextView)findViewById( R.id.counter );
                        counterTextView.setText( Integer.toString( counter ) );
                    } catch( RemoteException ex ) {
                        Log.e( LOG_TAG, ex.getMessage(), ex );
                    }
                }
            }
        });
        Button increaseCounterButton = (Button)findViewById( R.id.increasecounter );
        increaseCounterButton.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
                if( increaseCounterService != null )
                    try {
                        increaseCounterService.increaseCounter( 1 );
                    } catch( RemoteException ex ) {
                        Log.e( LOG_TAG, ex.getMessage(), ex );
                    }
            }
        });
    }

	private void initService() {
        if( increaseCounterServiceConnection == null ) {
            increaseCounterServiceConnection = new IncreaseCounterServiceConnection();
            Intent i = new Intent();
            i.setClassName( "aexp.oneway", "aexp.oneway.CounterService" );
            i.addCategory( CounterService.CATEGORY_IF_INCREASECOUNTER );
            bindService( i, increaseCounterServiceConnection, Context.BIND_AUTO_CREATE);
            Log.d( LOG_TAG, "bindService(): IncreaseCounter" );
        }

        if( counterServiceConnection == null ) {
            counterServiceConnection = new CounterServiceConnection();
            Intent i = new Intent();
            i.setClassName( "aexp.oneway", "aexp.oneway.CounterService" );
            i.addCategory( CounterService.CATEGORY_IF_COUNTER );
            bindService( i, counterServiceConnection, Context.BIND_AUTO_CREATE);

            Log.d( LOG_TAG, "bindService(): Counter" );
        } 

	}

	private void releaseService() {
	    if( counterServiceConnection != null ) {
	        unbindService( counterServiceConnection );	  
	        counterServiceConnection = null;
        }
	    if( increaseCounterServiceConnection != null ) {
	        unbindService( increaseCounterServiceConnection );	  
	        increaseCounterServiceConnection = null;
        }
	    Log.d( LOG_TAG, "unbindService()" );
	}

    class CounterServiceConnection implements ServiceConnection {
        public void onServiceConnected(ComponentName className, 
			IBinder boundService ) {
            counterService = Counter.Stub.asInterface((IBinder)boundService);
            Log.d( LOG_TAG,"CounterServiceConnection::onServiceConnected" );
        }

        public void onServiceDisconnected(ComponentName className) {
            counterService = null;
            Log.d( LOG_TAG,"CounterServiceConnection::onServiceDisconnected" );
        }
    };

    class IncreaseCounterServiceConnection implements ServiceConnection {
        public void onServiceConnected(ComponentName className, 
			IBinder boundService ) {
            increaseCounterService = IncreaseCounter.Stub.asInterface((IBinder)boundService);

            Log.d( LOG_TAG,"IncreaseCounterServiceConnection::onServiceConnected" );
        }

        public void onServiceDisconnected(ComponentName className) {
            increaseCounterService = null;
            Log.d( LOG_TAG,"IncreaseCounterServiceConnection::onServiceDisconnected" );
        }
    };


    private CounterServiceConnection counterServiceConnection;
    private IncreaseCounterServiceConnection increaseCounterServiceConnection;
    private Counter counterService;
    private IncreaseCounter increaseCounterService;
}
